#ifndef ASM6502_H
#define ASM6502_H


#include <map>
#include <vector>
#include <string>
#include <stdexcept>

#include "Assembler.h"


class Asm6502 : public Assembler {
public:
    enum addr_mode {
        ABS, // Absolute    $1234
        ABX, // Absolute,X  $1234,x
        ABY, // Absolute,y  $1234,y
        IMM, // Immediate   #$12
        IMP, // Implied
        IND, // Indirect    ($1234)
        INX, // Indirect,X  ($12,x)
        INY, // Indirect,Y  ($12),y
        REL, // Relative    $1234
        ZPG, // Zeropage    $12
        ZPX, // Zeropage,X  $12,x
        ZPY  // Zeropage,y  $12,y
    };
    
    Asm6502(const char *name, const char **lines);
    Asm6502(const char *name, std::vector<std::string>& lines);
    
    void init_addr_mode_map(void);
    addr_mode determine_addrmode(std::string& op, Assembler::TokenList& tlist);
    Assembler::Token get_argument_token(Asm6502::addr_mode mode, Assembler::TokenList& tlist);
    int get_op_code(std::string& op, addr_mode mode);
    bool valid_op(std::string& op);
    bool valid_mode(std::string& op, addr_mode mode);
    int get_mode_size(addr_mode mode);
        
    Chunk parse_opcode(std::string& op, Assembler::TokenList& tlist);

    //std::map<addr_mode, int>& get_mode_map(std::string& op);
    //std::vector<addr_mode> get_op_modes(std::string& op);
    
private:
    std::map<std::string, std::map<addr_mode, int> > m_op_map;
};


#endif
