#include "Assembler.h"


// empty chunk
Assembler::Chunk::Chunk()
: m_line(0), m_addr(0), m_size(0), m_ref_type(NONE) {
    
}


// chunk with no data
Assembler::Chunk::Chunk(int line, addr_t addr)
: m_line(line), m_addr(addr), m_size(0), m_ref_type(NONE) {

}


// chunk with a single byte of data
Assembler::Chunk::Chunk(int line, addr_t addr, int data)
: m_line(line), m_addr(addr), m_size(1), m_ref_type(NONE) {
    m_data.push_back(data);
}


// chunk with multiple bytes of data
Assembler::Chunk::Chunk(int line, addr_t addr, std::vector<int>& vector_data)
: m_line(line), m_addr(addr), m_size(vector_data.size()), m_ref_type(NONE) {
    m_data = vector_data;
}


// opcode chunk with an argument string
Assembler::Chunk::Chunk(int line, addr_t addr, int opcode, ref_type rt, std::string& label)
: m_line(line), m_addr(addr), m_ref_type(rt), m_label(label) {
    m_data.push_back(opcode);
    switch (rt) {
        
        case LABEL_REF:
        m_size = 3;
        break;
        
        case REL2_LABEL_REF:
        case LSB_LABEL_REF:
        case MSB_LABEL_REF:
        m_size = 2;
        break;
        
        default:
        throw "bad label reference type";
    }
}


std::vector<int> Assembler::Chunk::serialize_to_bytes(LabelDict& labeldict) {
    if (m_ref_type != NONE) {
        if (! labeldict.is_defined(m_label)) {
            throw "undefined label";
        }
    }
    
    if (m_ref_type == LABEL_REF) {
        // FIXME: LSB/MSB handling
        m_data.push_back(labeldict.get_addr(m_label) & 0xff);
        m_data.push_back(labeldict.get_addr(m_label) >> 8);
    } else if (m_ref_type == REL2_LABEL_REF) {
        addr_t offset = labeldict.get_addr(m_label) - (m_addr + 2);
        if (offset > 127 || offset < -128) {
            throw "relative offset out of range";
        }
        m_data.push_back(offset & 0xff);
    } else if (m_ref_type == LSB_LABEL_REF) {
        m_data.push_back(labeldict.get_addr(m_label) & 0xff);
    } else if (m_ref_type == MSB_LABEL_REF) {
        m_data.push_back(labeldict.get_addr(m_label) >> 8);
    }
    
    return m_data;
}
