#include "Assembler.h"


//#include "debug.h"


Assembler::Chunk Assembler::parse_command_org(Assembler::TokenList& tlist) {
    Token token;
    
    if (tlist.shift().get_type() != Token::SPACE) {
        throw "invalid argument";
    }
    token = tlist.shift();
    if (token.get_type() != Token::UINT16) {
        throw "invalid origin address";
    }
    m_curr_addr = token.get_num();
    if (tlist.shift().get_type() != Token::EOL) {
        throw "invalid argument";
    }
    return Assembler::Chunk(m_curr_line, m_curr_addr);
}
    

Assembler::Chunk Assembler::parse_command_align(Assembler::TokenList& tlist) {
    Token token;
    int align;
    
    if (tlist.shift().get_type() != Token::SPACE) {
        throw "invalid argument";
    }
    token = tlist.shift();
    if (! token.is_numeric()) {
        throw "invalid alignment value";
    }
    align = token.get_num();
    if (m_curr_addr % align) {
        m_curr_addr += align - (m_curr_addr % align);
    }
    
    if (tlist.shift().get_type() != Token::EOL) {
        throw "invalid argument";
    }
    return Assembler::Chunk(m_curr_line, m_curr_addr);
}
    

Assembler::Chunk Assembler::parse_command_byte(Assembler::TokenList& tlist) {
    Token token;
    std::vector<int> data;

    if (tlist.shift().get_type() != Token::SPACE) {
        throw "invalid argument";
    }
    do {
        while (tlist.front().get_type() == Token::SPACE) {
            tlist.shift();
        }
        token = tlist.shift();
        if (token.get_type() == Token::UINT8) {
            data.push_back(token.get_num());
        //} else if (token.get_type() == LTSTRING) {
        //} else if (token.get_type() == GTSTRING) {
        // FIXME: add support for label references
        } else if (token.get_type() == Token::QUOTSTRING) {
            std::string str = token.get_str();
            for (std::string::iterator si = str.begin(); si != str.end(); ++si) {
                data.push_back(*si);
            }
        } else {
            throw "invalid byte data";
        }
        token = tlist.shift();
        if (token.get_type() == Token::EOL) {
            return Assembler::Chunk(m_curr_line, m_curr_addr, data);
        }
        if (token.get_type() != Token::COMMA) {
            throw "invalid data separator";
        }
    } while (true);
}


Assembler::Chunk Assembler::parse_command_word(Assembler::TokenList& tlist) {
    Token token;
    std::vector<int> data;
    
    if (tlist.shift().get_type() != Token::SPACE) {
        throw "invalid argument";
    }
    do {
        while (tlist.front().get_type() == Token::SPACE) {
            tlist.shift();
        }
        token = tlist.shift();
        if (token.is_numeric()) {
            // FIXME: LSB/MSB handling
            data.push_back(token.get_num() & 0xff);
            data.push_back(token.get_num() >> 8);
        //} else if (token.get_type() == LTSTRING) {
        //} else if (token.get_type() == GTSTRING) {
        // FIXME: add support for label references
        } else {
            throw "invalid word data";
        }
        token = tlist.shift();
        if (token.get_type() == Token::EOL) {
            return Assembler::Chunk(m_curr_line, m_curr_addr, data);
        }
        if (token.get_type() != Token::COMMA) {
            throw "invalid data separator";
        }
    } while (true);
}


Assembler::Chunk Assembler::parse_command(std::string& cmd, Assembler::TokenList& tlist) {
#ifdef DEBUG_H
    debug_out() << "command: " << cmd << std::endl;
    debug_out() << format_token_list(tlist) << std::endl;
#endif
    if (cmd == ".org") {
        return parse_command_org(tlist);
    } else if (cmd == ".align") {
        return parse_command_align(tlist);
    } else if (cmd == ".byte") {
        return parse_command_byte(tlist);
    } else if (cmd == ".word") {
        return parse_command_word(tlist);
    } else {
        throw "unknown command";
    }
    
    return Assembler::Chunk(m_curr_line, m_curr_addr);
}


