#include "Assembler.h"


void Assembler::LabelDict::define(std::string& name, int size, addr_t addr) {
    if (is_defined(name)) {
        throw "label redefined";
    }
    m_lab_addr[name] = addr;
    m_lab_size[name] = size;
}

void Assembler::LabelDict::define(const char *name, int size, addr_t addr) {
    std::string str = name;
    
    define(str, size, addr);
}


bool Assembler::LabelDict::is_defined(std::string& name) {
    return m_lab_addr.count(name) > 0;
}


int Assembler::LabelDict::get_size(std::string& name) {
    if (is_defined(name)) {
        return m_lab_size[name];
    } else {
        throw "undefined label";
    }
}


addr_t Assembler::LabelDict::get_addr(std::string& name) {
    if (is_defined(name)) {
        return m_lab_addr[name];
    } else {
        throw "undefined label";
    }
}

addr_t Assembler::LabelDict::get_addr(const char *name) {
    std::string str = name;
    
    return get_addr(str);
}


void Assembler::LabelDict::clear(void) {
    m_lab_addr.clear();
    m_lab_size.clear();
}


std::vector<std::string> Assembler::LabelDict::get_label_list(void) {
    std::vector<std::string> label_list;
    std::map<std::string,addr_t>::iterator mi;
    
    for (mi = m_lab_addr.begin(); mi != m_lab_addr.end(); ++mi) {
        label_list.push_back(mi->first);
    }
    return label_list;
}
