#include "Assembler.h"


void Assembler::parse_labeldef(Assembler::Token label, Assembler::TokenList& tlist) {
    Assembler::Token t;
    
    t = tlist.shift();
    if (t.get_type() == Assembler::Token::COLON) {
        labeldict.define(label.get_str(), 16, m_curr_addr);
    } else {
        if (t.get_type() == Assembler::Token::SPACE) {
            t = tlist.shift();
        }
        if (t.get_type() != Assembler::Token::EQUAL) {
            throw "malformed label definition";
        }
        t = tlist.shift();
        if (t.get_type() == Assembler::Token::SPACE) {
            t = tlist.shift();
        }
        if (t.get_type() == Assembler::Token::UINT8) {
            labeldict.define(label.get_str(), 8, t.get_num());
        } else if (t.get_type() == Assembler::Token::UINT16) {
            labeldict.define(label.get_str(), 16, t.get_num());
        } else if (t.get_type() == Assembler::Token::STRING) {
            if (! labeldict.is_defined(t.get_str())) {
                throw "undefined label";
            }
            labeldict.define(label.get_str(), labeldict.get_size(t.get_str()), labeldict.get_addr(t.get_str()));
        } else if (t.get_type() == Assembler::Token::LTSTRING) {
            if (! labeldict.is_defined(t.get_str())) {
                throw "undefined label";
            }
            labeldict.define(label.get_str(), 8, labeldict.get_addr(t.get_str()) & 0xff);
        } else if (t.get_type() == Assembler::Token::GTSTRING) {
            if (! labeldict.is_defined(t.get_str())) {
                throw "undefined label";
            }
            labeldict.define(label.get_str(), 8, labeldict.get_addr(t.get_str()) >> 8);
        }
    }
    if (tlist.shift().get_type() != Assembler::Token::EOL) {
        throw "unexpected data after label definition";
    }
}


Assembler::Chunk Assembler::parse_instruction(Assembler::TokenList& tlist) {
    Assembler::Token t;
    
    t = tlist.shift();
    if (t.get_type() == Assembler::Token::STRING) {
        return parse_opcode(t.get_str(), tlist);
    } else if (t.get_type() == Assembler::Token::DOTSTRING) {
        return parse_command(t.get_str(), tlist);
    } else {
        throw "syntax error";
    }
}


Assembler::Chunk Assembler::parse_line(Assembler::TokenList& tlist) {
    Assembler::Token t;
    
    t = tlist.shift();
    
    // FIXME: parse labels first to allow labels before instructions
    if (t.get_type() == Assembler::Token::EOL) {
        return Assembler::Chunk(m_curr_line, m_curr_addr);
    } else if (t.get_type() == Assembler::Token::STRING) {
        parse_labeldef(t, tlist);
        return Assembler::Chunk(m_curr_line, m_curr_addr);
    } else if (t.get_type() == Assembler::Token::SPACE) {
        return parse_instruction(tlist);
    } else {
        throw "syntax error";
    }
}
