#include "Asm6502.h"


Assembler::Chunk Asm6502::parse_opcode(std::string& op, Assembler::TokenList& tlist) {
    addr_mode mode;
    int opcode;
    int size;
    
    if (! valid_op(op)) {
        throw "unknown opcode";
    }
    
    mode = determine_addrmode(op, tlist);
    opcode = get_op_code(op, mode);
    size = get_mode_size(mode);
    
    if (mode == Asm6502::IMP) {
        return Assembler::Chunk(m_curr_line, m_curr_addr, opcode);
        
    } else {
        Token token = get_argument_token(mode, tlist);
        Token::token_type tt = token.get_type();
        
        if (token.is_numeric()) {
            int num = token.get_num();
            
            std::vector<int> data;
            data.push_back(opcode);
        
            if (tt == Token::UINT8) {
                data.push_back(num);
            } else if (tt == Token::UINT16) {
                if (mode == Asm6502::REL) {
                    addr_t offset = num - (m_curr_addr + 2);
                    if (offset > 127 || offset < -128) {
                        throw "relative offset out of range";
                    }
                    data.push_back(offset & 0xff);
                } else {
                    data.push_back(num & 0xff);
                    data.push_back(num >> 8);
                }
            }
            return Assembler::Chunk(m_curr_line, m_curr_addr, data);
            
        } else {
            std::string str = token.get_str();
            Assembler::Chunk::ref_type rt;
            
            if (tt == Token::LTSTRING) {
                rt = Assembler::Chunk::LSB_LABEL_REF;
            } else if (tt == Token::GTSTRING) {
                rt = Assembler::Chunk::MSB_LABEL_REF;
            } else if (tt == Token::STRING) {
                if (mode == Asm6502::REL) {
                    rt = Assembler::Chunk::REL2_LABEL_REF;
                } else if (get_mode_size(mode) == 2) {
                    rt = Assembler::Chunk::LSB_LABEL_REF;
                } else {
                    rt = Assembler::Chunk::LABEL_REF;
                }
            } else {
                throw "unknown argument token";
            }
            return Assembler::Chunk(m_curr_line, m_curr_addr, opcode, rt, str);
        }
    }
}
