#include "File.h"


/*
class File {
public:
    File(const char *filename, const char *mode);
    ~File();
    size_t read(char *buffer, size_t amount);
    size_t write(char *buffer, size_t amount);
};
*/


File::File(std::string filename, file_mode mode, binary_mode binary)
: m_mode(mode), m_binary(binary) {
    std::ios_base::openmode fs_mode;
    
    try {
        // enable exception throwing
        m_fs.exceptions(std::ios_base::failbit | std::ios_base::badbit);
        
        if (mode == READ) {
            fs_mode = std::ios_base::in;
        } else if (mode == WRITE) {
            fs_mode = std::ios_base::out;
        } else {
            throw FileException("bad file mode");
        }
        if (binary == BINARY) {
            fs_mode |= std::ios_base::binary;
        } else if (binary != TEXT) {
            throw FileException("bad binary mode");
        }
        
        m_fs.open(filename.c_str(), fs_mode);
    }
    catch (std::fstream::failure e) {
        throw FileException(e.what());
    }
}


File::~File() {
    // disable exception throwing before closing
    m_fs.exceptions(std::ios_base::goodbit);
    m_fs.close();
}


std::streamsize File::read(char *buffer, std::streamsize amount) {
    if (m_mode != READ) {
        throw FileException("file was not opened for reading");
    }
    m_fs.read(buffer, amount);
    return m_fs.gcount();
}


std::streamsize File::write(char *buffer, std::streamsize amount) {
    if (m_mode != WRITE) {
        throw FileException("file was not opened for writing");
    }
    m_fs.write(buffer, amount);
    return amount;
}


std::streamsize File::putchar(char c) {
    if (m_mode != WRITE) {
        throw FileException("file was not opened for writing");
    }
    m_fs.put(c);
    return 1;
}


std::streamsize File::data_left(void) {
    if (m_mode == READ) {
        std::streampos start_pos = m_fs.tellg();
        m_fs.seekg(0, std::ios_base::end);
        std::streamsize left = m_fs.tellg() - start_pos;
        m_fs.seekg(start_pos, std::ios_base::beg);
        return left;
    } else {
        throw FileException("file was not opened for reading");
    }
}


std::vector<std::string> File::read_lines(void) {
    std::vector<std::string> lines;
    
    throw FileException("File::read_lines unimplemented");
}
