#ifndef FILE_H
#define FILE_H


#include <string>
#include <vector>
#include <fstream>


class FileException : public std::ios_base::failure {
public:
    FileException(const std::string& msg) : std::ios_base::failure(msg) {};
};


class File {
public:
    enum file_mode {
        READ,
        WRITE
    };
    enum binary_mode {
        BINARY,
        TEXT
    };
    File(std::string filename, file_mode mode, binary_mode binary);
    ~File();
    std::streamsize read(char *buffer, std::streamsize amount);
    std::streamsize write(char *buffer, std::streamsize amount);
    std::streamsize putchar(char c);
    std::streamsize data_left(void);
    std::vector<std::string> read_lines(void);
    
private:
    std::fstream m_fs;
    file_mode m_mode;
    binary_mode m_binary;
};


#endif /* end of include guard: FILE_H */
