#ifndef IMAGE_H
#define IMAGE_H


#include <memory>
#include <stdexcept>

#include "Memory.h"


class ImageException : public std::runtime_error {
public:
    ImageException(const std::string& what_arg) : std::runtime_error(what_arg) {};
};


class Image {
public:
    Image(const char *filename);
    virtual ~Image() {};
    
    virtual int load(void);
    virtual void unpack(void) = 0;
    virtual void assemble(void) = 0;
    
    Memory m_mem;
    const char *m_filename;
};


class ImageFactory {
public:
    std::auto_ptr<Image> get_image(const char *filename);
};


/*
class HiresBitmapImage : public Image {
public:
    
};
*/


class MulticolorBitmapImage : public Image {
public:
    MulticolorBitmapImage(const char *filename) : Image(filename) {};
    void assemble(void);
    virtual void unpack(void) = 0;
};


class ImgMCBKoala : public MulticolorBitmapImage {
public:
    ImgMCBKoala(const char *filename) : MulticolorBitmapImage(filename) {};
    void unpack(void);
};


#endif /* end of include guard: IMAGE_H */
