#ifndef MEMORY_H
#define MEMORY_H


#include <stdexcept>

#include "types.h"


#define MAXMEM 0x10000


class MemoryException : public std::range_error {
public:
    MemoryException(const std::string& what_arg) : std::range_error(what_arg) {};
};


class Memory {
public:
    Memory(byte_t clear_value = 0);
    
    void clear(byte_t value = 0);
    void clear(addr_t start, addr_t end, byte_t value = 0);
    void set(addr_t addr, byte_t value);
    byte_t get(addr_t addr);
    
    void copy(addr_t dest_addr, addr_t src_addr, int len);
    void copy(addr_t dest_addr, Memory& src_mem, addr_t src_addr, int len);
    
    void read(byte_t *dest_addr, addr_t src_addr, int len);
    void write(addr_t dest_addr, byte_t *src_addr, int len);
    
    bool is_addr_valid(addr_t addr);
    bool is_range_valid(addr_t start, addr_t end);
    
    addr_t get_highest(void) { return m_highest; };
    addr_t get_lowest(void) { return m_lowest; };

    void set_highest(addr_t addr);
    void set_lowest(addr_t addr);
    void set_hi_lo(addr_t start, addr_t end);
    
    byte_t *get_valid_range_ptr(addr_t start, addr_t end);
    
private:
    addr_t m_highest;
    addr_t m_lowest;
    byte_t m_mem[MAXMEM];
    int m_size;
};



#endif /* end of include guard: MEMORY_H */
