#include "debug.h"


std::string format_token(Assembler::Token token) {
    std::stringstream ss;
    
    switch (token.get_type()) {
        case Assembler::Token::SPACE:
        ss << "[SPACE] ";
        break;
        case Assembler::Token::STRING:
        ss << "[STRING: " << token.get_str() << "] ";
        break;
        case Assembler::Token::QUOTSTRING:
        ss << "[QUOTSTRING: " << token.get_str() << "] ";
        break;
        case Assembler::Token::DOTSTRING:
        ss << "[DOTSTRING: " << token.get_str() << "] ";
        break;
        case Assembler::Token::LTSTRING:
        ss << "[LTSTRING: " << token.get_str() << "] ";
        break;
        case Assembler::Token::GTSTRING:
        ss << "[GTSTRING: " << token.get_str() << "] ";
        break;
        case Assembler::Token::UINT8:
        ss << "[UINT8: " << format_hex(token.get_num(), 2) << "] ";
        break;
        case Assembler::Token::UINT16:
        ss << "[UINT16: " << format_hex(token.get_num(), 4) << "] ";
        break;
        case Assembler::Token::HASH:
        ss << "[HASH] ";
        break;
        case Assembler::Token::COLON:
        ss << "[COLON] ";
        break;
        case Assembler::Token::EQUAL:
        ss << "[EQUAL] ";
        break;
        case Assembler::Token::COMMA:
        ss << "[COMMA] ";
        break;
        case Assembler::Token::LPAREN:
        ss << "[LPAREN] ";
        break;
        case Assembler::Token::RPAREN:
        ss << "[RPAREN] ";
        break;
        case Assembler::Token::EOL:
        ss << "[EOL] ";
        break;
        default:
        ss << token.get_type();
        //throw AsmError("unknown token");
    }
    return ss.str();
}


std::string format_token_list(Assembler::TokenList& tlist) {
    std::stringstream ss;
    Assembler::TokenList::iterator tli;
    
    for (tli = tlist.begin(); tli != tlist.end(); ++tli) {
        ss << format_token(*tli);
    }
    
    return ss.str();
}


void debug_print_data(int line, addr_t addr, std::vector<int>& data) {
    std::vector<int>::iterator ii;
    
    std::cout << line << ":" << format_hex(addr, 4) << "  ";
    for (ii = data.begin(); ii != data.end(); ++ii) {
        std::cout << " " << format_hex(*ii, 2);
    }
    std::cout << std::endl;
}


