#include <iostream>

#include "format.h"
#include "output.h"
#include "File.h"
#include "Image.h"
#include "Compressor.h"
#include "generate.h"


static void save_program(const char *output_filename, Memory& mem, addr_t start, addr_t end) {
    int len = end - start + 1;
    File outfile(output_filename, File::WRITE, File::BINARY);
    outfile.putchar(start & 0xff);
    outfile.putchar(start >> 8);
    outfile.write((char *) mem.get_valid_range_ptr(start, end), len);
}


void generate_executable(const char *input_filename, const char *output_filename) {
    ImageFactory img_factory;
    CompressorFactory comp_factory;
    Memory mem;
    
    info_out() << "Unpacking and assembling " << input_filename << std::endl;
    // load and unpack the image
    std::auto_ptr<Image> image(img_factory.get_image(input_filename));
    image->load();
    image->unpack();
    image->assemble();
    verbose_out() << "Executable uses " << format_hex(image->m_mem.get_lowest(), 4)
               << " to " << format_hex(image->m_mem.get_highest(), 4)
               << std::endl;
    mem = image->m_mem;
        
    // compress
    std::auto_ptr<Compressor> compressor(comp_factory.get_compressor(Compressor::LZSS));
    info_out() << "Compressing with " << compressor->get_name() << std::endl;
    mem = compressor->compress(mem, mem.get_lowest(), mem.get_highest(), 0x0a00);
    
    // add sys line
    compressor->add_sys(mem);
    
    info_out() << "Saving as " << output_filename << std::endl;
    save_program(output_filename, mem, mem.get_lowest(), mem.get_highest());
}
