#include <iostream>
#include <vector>
#include <string>

#include "global.h"
#include "File.h"
#include "format.h"
#include "Asm6502.h"


static const char program_executable[] = "test_assembler";

static const char *input_filename;
static const char *output_filename;


void assemble(std::string infname, std::string outfname) {
    File infile(infname, File::READ, File::TEXT);
    std::vector<std::string> source = infile.read_lines();
    
    Asm6502 assembler(infname.c_str(), source);
    Memory mem;
    
    try {
        assembler.assemble(0, mem);
        if (global_pref.verbosity >= GlobalPref::VERBOSE) {
            std::vector<std::string> labels = assembler.labeldict.get_label_list();
            for (std::vector<std::string>::iterator vsi = labels.begin();
                 vsi != labels.end();
                 ++vsi) {
                std::string label = *vsi;
                addr_t addr = assembler.labeldict.get_addr(label);
                int size = assembler.labeldict.get_size(label);
                if (size == 8) {
                    verbose_out() << format_string(label, 16) << "\t  " << format_hex(addr, 2) << std::endl;
                } else {
                    verbose_out() << format_string(label, 16) << "\t" << format_hex(addr, 4) << std::endl;
                }
            }
        }
        info_out() << "Lowest: " << format_hex(mem.get_lowest(), 4)
                   << ", highest: " << format_hex(mem.get_highest(), 4) << std::endl;
    }
    catch (AsmError& e) {
        std::cerr << e.what() << std::endl;
    }
}


static bool parse_args(int argc, const char *argv[]) {
    if (argc != 3) {
        return false;
    }
    input_filename = argv[1];
    output_filename = argv[2];
    return true;
}


static void print_usage(void) {
    std::cout << "Usage: " << program_executable << " infile.asm outfile.prg" << std::endl;
}


int main(int argc, const char *argv[]) {
    if (!parse_args(argc, argv)) {
        print_usage();
        return 1;
    }

    assemble(input_filename, output_filename);
    
    return 0;
}
