#include <iostream>

#include "global.h"
#include "File.h"
#include "Image.h"
#include "Compressor.h"
#include "generate.h"


static const char program_executable[] = "xi";

static const char *input_filename;
static const char *output_filename;


static bool parse_args(int argc, const char *argv[]) {
    if (argc != 3) {
        return false;
    }
    input_filename = argv[1];
    output_filename = argv[2];
    return true;
}


static void print_usage(void) {
    std::cout << "Usage: " << program_executable << " infile.koa outfile.prg" << std::endl;
}


int main(int argc, const char *argv[]) {
    if (!parse_args(argc, argv)) {
        print_usage();
        return 1;
    }

    try {
        generate_executable(input_filename, output_filename);
    }
    catch (ImageException& e) {
        std::cerr << "Loading " << input_filename << " failed: " << e.what() << std::endl;
        return 2;
    }
    catch (CompressorException& e) {
        std::cerr << "Compressing " << input_filename << " failed: " << e.what() << std::endl;
        return 2;
    }
    catch (FileException& e) {
        std::cerr << "Saving " << output_filename << " failed: " << e.what() << std::endl;
        return 2;
    }
    catch (const char *msg) {
        std::cerr << "Uncaught exception: " << msg << std::endl;
        return 2;
    }
    
    return 0;
}
